# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .audio import (
    AudioResource,
    AsyncAudioResource,
    AudioResourceWithRawResponse,
    AsyncAudioResourceWithRawResponse,
    AudioResourceWithStreamingResponse,
    AsyncAudioResourceWithStreamingResponse,
)
from .speech import (
    SpeechResource,
    AsyncSpeechResource,
    SpeechResourceWithRawResponse,
    AsyncSpeechResourceWithRawResponse,
    SpeechResourceWithStreamingResponse,
    AsyncSpeechResourceWithStreamingResponse,
)
from .voices import (
    VoicesResource,
    AsyncVoicesResource,
    VoicesResourceWithRawResponse,
    AsyncVoicesResourceWithRawResponse,
    VoicesResourceWithStreamingResponse,
    AsyncVoicesResourceWithStreamingResponse,
)
from .translations import (
    TranslationsResource,
    AsyncTranslationsResource,
    TranslationsResourceWithRawResponse,
    AsyncTranslationsResourceWithRawResponse,
    TranslationsResourceWithStreamingResponse,
    AsyncTranslationsResourceWithStreamingResponse,
)
from .transcriptions import (
    TranscriptionsResource,
    AsyncTranscriptionsResource,
    TranscriptionsResourceWithRawResponse,
    AsyncTranscriptionsResourceWithRawResponse,
    TranscriptionsResourceWithStreamingResponse,
    AsyncTranscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "SpeechResource",
    "AsyncSpeechResource",
    "SpeechResourceWithRawResponse",
    "AsyncSpeechResourceWithRawResponse",
    "SpeechResourceWithStreamingResponse",
    "AsyncSpeechResourceWithStreamingResponse",
    "VoicesResource",
    "AsyncVoicesResource",
    "VoicesResourceWithRawResponse",
    "AsyncVoicesResourceWithRawResponse",
    "VoicesResourceWithStreamingResponse",
    "AsyncVoicesResourceWithStreamingResponse",
    "TranscriptionsResource",
    "AsyncTranscriptionsResource",
    "TranscriptionsResourceWithRawResponse",
    "AsyncTranscriptionsResourceWithRawResponse",
    "TranscriptionsResourceWithStreamingResponse",
    "AsyncTranscriptionsResourceWithStreamingResponse",
    "TranslationsResource",
    "AsyncTranslationsResource",
    "TranslationsResourceWithRawResponse",
    "AsyncTranslationsResourceWithRawResponse",
    "TranslationsResourceWithStreamingResponse",
    "AsyncTranslationsResourceWithStreamingResponse",
    "AudioResource",
    "AsyncAudioResource",
    "AudioResourceWithRawResponse",
    "AsyncAudioResourceWithRawResponse",
    "AudioResourceWithStreamingResponse",
    "AsyncAudioResourceWithStreamingResponse",
]
