# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["FilePurpose"]

FilePurpose: TypeAlias = Literal[
    "fine-tune", "eval", "eval-sample", "eval-output", "eval-summary", "batch-generated", "batch-api"
]
