# Manually added to minimize breaking changes from V1
from ._constants import (
    MAX_RETRY_DELAY as MAX_RETRY_DELAY,
    DEFAULT_MAX_RETRIES,
    INITIAL_RETRY_DELAY as INITIAL_RETRY_DELAY,
)
from .lib.constants import (
    MIN_SAMPLES as MIN_SAMPLES,
    DISABLE_TQDM as DISABLE_TQDM,
    MAX_IMAGE_BYTES as MAX_IMAGE_BYTES,
    NUM_BYTES_IN_GB as NUM_BYTES_IN_GB,
    MAX_FILE_SIZE_GB as MAX_FILE_SIZE_GB,
    MIN_PART_SIZE_MB as MIN_PART_SIZE_MB,
    DOWNLOAD_BLOCK_SIZE as DOWNLOAD_BLOCK_SIZE,
    MAX_MULTIPART_PARTS as MAX_MULTIPART_PARTS,
    TARGET_PART_SIZE_MB as TARGET_PART_SIZE_MB,
    MAX_CONCURRENT_PARTS as MAX_CONCURRENT_PARTS,
    MAX_IMAGES_PER_EXAMPLE as MAX_IMAGES_PER_EXAMPLE,
    MULTIPART_THRESHOLD_GB as MULTIPART_THRESHOLD_GB,
    MAX_BASE64_IMAGE_LENGTH as MAX_BASE64_IMAGE_LENGTH,
    MULTIPART_UPLOAD_TIMEOUT as MULTIPART_UPLOAD_TIMEOUT,
    PARQUET_EXPECTED_COLUMNS as PARQUET_EXPECTED_COLUMNS,
    REQUIRED_COLUMNS_MESSAGE as REQUIRED_COLUMNS_MESSAGE,
    JSONL_REQUIRED_COLUMNS_MAP as JSONL_REQUIRED_COLUMNS_MAP,
    POSSIBLE_ROLES_CONVERSATION as POSSIBLE_ROLES_CONVERSATION,
    DatasetFormat as DatasetFormat,
)

TIMEOUT_SECS = 600
MAX_SESSION_LIFETIME_SECS = 180
MAX_CONNECTION_RETRIES = 2
MAX_RETRIES = DEFAULT_MAX_RETRIES

BASE_URL = "https://api.together.xyz/v1"
